#!/bin/bash

# gen_yearvec.bash
#
# Call gendaylit and genskyvec for each hour in a weather file.
# gendaylit errors are handled and logged in an error file.
#
# (c) Jan 2010, Axel Jacobs

# Input: the EPW weather file
wea="weather"
epw="$wea/GBR_London.Gatwick.037760_IWEC.epw"
# We keep all gendaylit errors to ponder about them...
err="$wea/gatwick.err"
# Output: time series of sky vectors
res="$wea/gatwick_year.vecs"
# Output: month, day, decimal hour
dtm="$wea/timedate.csv"
rm -f $err $res $dtm

nullvec='0'
# Adjust of you use gendaylit with anything other than -m 1
for i in {1..145}; do
    nullvec="$nullvec 0"
done
# Sky vector from the previous time step
prevvec=""

# Extract some information from the EPW header:
# Location is the first line the EPW header
loc=$(head -1 $epw)
lat=$(echo $loc |cut -d',' -f7)
longitude=$(echo $loc |cut -d',' -f8)
# EPW lists longitude + to the east, Radiance + to the west
lon=$(echo $longitude |rcalc -e '$1=-1*$1')
meridian=$(echo $loc |cut -d',' -f9)
# EPW lists time zone in hours, but we need degrees
mer=$(echo $meridian |rcalc -e '$1=$1*-15')
#echo "lat/lon: $lat/$lon, mer: $mer"

tail -n +9 $epw |rcalc -t, \
        -e '$1=$2;$2=$3;$3=$4-1+($5/2)/60;$4=$15;$5=$16' \
        |tr ',' '\t' |while read line; do

    datetime=$(echo $line |cut -d' ' -f1-3)
    dirnorm="$(echo $line |cut -d' ' -f4)"
    diffhor="$(echo $line |cut -d' ' -f5)"
    radiation="$dirnorm $diffhor"

    # Out put date and time to separate file
    echo "$datetime" |tr ' ' '\t' >> $dtm
    # Output null vector at night
    if [ "$radiation" == '0 0' ]; then
        skyvec=$nullvec
    else
        # Assemble gendaylit command line
        gdlcmd="$datetime -a $lat -o $lon -m $mer -W $radiation"
        # Test run. See if gendaylit succeeds
        gendaylit $gdlcmd >/dev/null 2>&1
        # Did something go wrong?
        if [ $? -eq 0 ]; then
            # All fine. Generate sky vector, extract first column and transpose.
            skyvec=$(gendaylit $gdlcmd \
                    |genskyvec -c 1 1 1 -m 1 |cut -f1 |tr '\n' ' ')
            prevvec=$skyvec
        elif [ $dirnorm -eq 0 ]; then
            # Error: gendaylit dropped out.
            # Most errors in the Gatwick file occur at dawn or nightfall
            # when the direct normal radiance is zero. Assume it's night.
            # For Gatwick, this happens 190 times.
            echo "$datetime: $radiation (assuming night time)" >> $err
            skyvec=$nullvec
        else
            # This is a serious gendaylit error. Re-use previous sky vector.
            # For Gatwick, this doesn't happen at all.
            skyvec=$prevvec
            echo "$datetime: $radiation (re-using previous sky vector)" >> $err
        fi
    fi
    echo $skyvec >> $res
done

#EOF
